Package: augere.screen
Version: 0.99.2
Date: 2026-04-12
Title: Automatic Generation of Functional Screen Analyses
Description: 
    Implements pipelines for generating functional screen analysis reports in the augere framework.
    This uses voom to test for differential abundance of barcodes with consolidation into gene-level statistics.
    Each pipeline function generates a self-contained Rmarkdown report with all of the steps required to reproduce the analysis.
Authors@R: person("Aaron", "Lun", role=c("cre", "aut"), email="infinite.monkeys.with.keyboards@gmail.com", comment=c(ORCID="0000-0002-3564-4813"))
License: MIT + file LICENSE
Imports: stats, limma, edgeR, metapod, S4Vectors, augere.core,
        augere.de
Suggests: testthat, knitr, rmarkdown, BiocStyle, SummarizedExperiment
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
URL: https://github.com/augere-bioinfo/augere.screen
BugReports: https://github.com/augere-bioinfo/augere.screen/issues
biocViews: WorkflowManagement, ReportWriting, GeneTarget,
        FunctionalGenomics
git_url: https://git.bioconductor.org/packages/augere.screen
git_branch: devel
git_last_commit: 5df23e6
git_last_commit_date: 2026-04-11
Repository: Bioconductor 3.24
Date/Publication: 2026-05-07
NeedsCompilation: no
Packaged: 2026-05-07 20:20:14 UTC; biocbuild
Author: Aaron Lun [cre, aut] (ORCID: <https://orcid.org/0000-0002-3564-4813>)
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
